﻿using PrecastConcretePlantContracts.BindingModels;
using PrecastConcretePlantContracts.SearchModels;
using PrecastConcretePlantContracts.StoragesContracts;
using PrecastConcretePlantContracts.ViewModels;
using PrecastConcretePlantFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace PrecastConcretePlantFileImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataFileSingleton source;

        public OrderStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            var element = source.Orders.FirstOrDefault(x => x.Id == model.Id);

            if (element != null)
            {
                source.Orders.Remove(element);
                source.SaveOrders();

                return ViewAddReinforcedName(element.GetViewModel);
            }

            return null;
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            return ViewAddReinforcedName(order.GetViewModel);
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {

            if (model.Id.HasValue)
            {
                return source.Orders.Where(x => x.Id == model.Id).Select(x => ViewAddReinforcedName(x.GetViewModel)).ToList();
            }
            else if (model.DateFrom != null && model.DateTo != null)
            {
                return source.Orders.Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo).Select(x => ViewAddReinforcedName(x.GetViewModel)).ToList();
            }
            else if (model.ClientId.HasValue)
            {
                return source.Orders.Where(x => x.ClientId == model.ClientId).Select(x => ViewAddReinforcedName(x.GetViewModel)).ToList();
            }
            return new();
        }

        public List<OrderViewModel> GetFullList()
        {
            return source.Orders.Select(x => ViewAddReinforcedName(x.GetViewModel)).ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = source.Orders.Count > 0 ? source.Orders.Max(x => x.Id) + 1 : 1;
            var newOrder = Order.Create(model);

            if (newOrder == null)
            {
                return null;
            }

            source.Orders.Add(newOrder);
            source.SaveOrders();

            return ViewAddReinforcedName(newOrder.GetViewModel);
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);

            if (order == null)
            {
                return null;
            }

            order.Update(model);
            source.SaveOrders();

            return ViewAddReinforcedName(order.GetViewModel);
        }
        private OrderViewModel ViewAddReinforcedName(OrderViewModel model)
        {
            var selectedReinforced = source.Reinforceds.FirstOrDefault(x => x.Id == model.ReinforcedId);
            model.ReinforcedName = selectedReinforced?.ReinforcedName;
            return model;
        }
    }
}
